           System for Lab Notes and Literate Programming v2
                            Opened: 8/5/12

@@@
\tableofcontents
@@@

* Introduction

When working on simulations and computer-based modeling/experiments,
it is necessary to organize data, keep track of information, and
document ideas.  However, no quality system exists for this type of
work.

The script contained herein describes a system that allows one to keep
a lab notebook.  It uses a format that is attractive and readable as a
plain text file, yet also looks good typeset with LaTeX as a pdf.  

This text format is dynamic.  It continues to evolve to fit the needs
at hand.  While it will try to maintain backwards compatibility, this
is not always possible.  Additions and deletions will be documented in
this file.

UPDATE (10/7/12): I realized recently how useful it would be to have a
system that I can use for lab reports as well.  I have updated the
document/code accordingly.

** Emacs mode

I have written an emacs mode ("notes-mode") to make working with notes
files easier in emacs.  It can be installed by moving the file
"notes-mode.el" into the ".emacs.d/site-lisp" directory, and adding
the following lines to your ".emacs" file:

> (add-to-list 'load-path "~/.emacs.d/site-lisp")
> (let ((default-directory "~/.emacs.d/site-lisp/"))
>      (normal-top-level-add-subdirs-to-load-path))
> (require 'notes-mode)

Alternatively, you can place it in any directory in emacs' load path
and just add the following to your ".emacs" file:

> (require 'notes-mode)

This mode is based on org-mode.  Most of the features of org-mode
don't make a lot of sense with notes-mode, so for now, they may be
ignored.  I may try later to only import the features of org-mode that
are needed.

* Format

(To be written later)

* Implementation

The following Perl code is a working implementation of this format.

** Initialization

<<latexify-notes.pl>>
===>
#!/usr/bin/perl -w
use Regexp::Grammars;

if ($#ARGV != 0) {
    die "\nUsage: ./latexify-notes.pl [FILE]\nFILE is the formatted file to be converted to latex format";
}
<===

** Global variables


The "@tokens" variable stores the list of tokens as they are detected
by the regex.  They are stored here in a simple list, where
"___TOKENn___" corresponds to tokens[n].

===>
@tokens = ();
<===

"content" is the text that is read from the file.  It will be
processed in-place, i.e. after processing, content will contain the
output string.  All operations performed in this script are for the
purpose of manipulating the "content" variable.

===>
#$content = do{ local $/; <DATA>};
$filename = $ARGV[0];
local $/ = undef;
open(FILE, $filename);
$content = <FILE>;
close(FILE);
<===


The first line of the document is separated and stored in "title".

===>
($title, $content) = split("\n", $content, 2);
$content = "\n" . $content . "\n";
do { no Regexp::Grammars; $title =~ s/^\s*//; $title =~ s/\s*$//};
<===

The "%properties" hash stores properties as they are accumulated in
parsing the document.  These can be set within the document in "Key:
Value" format.  Note that both "key" and "value" must be alphanumerics
only.  All properties besides "By", "Opened", "Closed", "Type", and
"Project" are currently ignored in the standard note-taking mode.
When "Type" is set to "Report", the properties "Course" and
"Instructor" are also recognized.

===>
%properties = ('By' => "Maxwell W. Shinn", 
               'Type' => "Note", 
               'Spacing' => "Single");
<===

The ID is just the filename without the ".n", and the extension
encompasses everything after the first period.

===>
$id = "";
$suffix = "";
$ext = "";
do { 
    no Regexp::Grammars; 
    if ($filename =~ m/^.*?([^\/\\]+)\.([nr])$/) {
        $id = $1;
        $suffix = $2;
        if ($suffix eq "r") { $properties{'Type'} = "Report"; }
        if ($id =~ m/\./) { $ext = (split("\\.", $id, 2))[1]; }
    }
    else {
        die "Invalid file name.";
    }
};
<===

Here, we create the machinery necessary for backreferences to labels.
The variable "%labeltexts" is a hash that describes all of the labels
in the document.  The text to display is indexed by the label name.
For instance, a label called "dispequation" would be called "Equation
1" in the document, so the text would be "Equation".  Thus, we would
have "('dispequation' => 'Equation ')" as the expected value of this
hash.  Similarly, "%labeltypes" includes the short latex name that
describes the type indexed by the same label as "%labeltexts".
Equations are "eq", figures are "fig", and tables are "tab".

===>
%labeltexts = ();
%labeltypes = ();
<===

Additionally, it is possible to automatically create a bibliography by
specifying "auto" as the source for the bibliography.  This variable
holds the full text for the ".bib" file, generated as each call to a
DOI is created.

===>
$bibtex_entries = "";
<===

** Substitutions

These define a list of simple substitutions.  The keys are replaced
with the values in "%replacements".  In "%replacements_math", they
text is enclosed in an ensuremath command when it is replaced.

===>
%replacements_math = (
    "α"   => "\\alpha",
    "β"   => "\\beta",
    "γ"   => "\\gamma",
    "δ"   => "\\delta",
    "ε"   => "\\epsilon",
    "ζ"   => "\\zeta",
    "η"   => "\\eta",
    "θ"   => "\\theta",
    "ι"   => "\\iota",
    "κ"   => "\\kappa",
    "λ"   => "\\lambda",
    "μ"   => "\\mu",
    "ν"   => "\\nu",
    "ξ"   => "\\xi",
    "ο"   => "\\omicron",
    "π"   => "\\pi",
    "ρ"   => "\\rho",
    "σ"   => "\\sigma",
    "ς"   => "\\varsigma",
    "τ"   => "\\tau",
    "υ"   => "\\upsilon",
    "φ"   => "\\varphi",
    "χ"   => "\\chi",
    "ψ"   => "\\psi",
    "ω"   => "\\omega",
    "Γ"   => "\\Gamma",
    "Δ"   => "\\Delta",
    "Θ"   => "\\Theta",
    "Λ"   => "\\Lambda",
    "Ξ"   => "\\Xi",
    "Π"   => "\\Pi",
    "Σ"   => "\\Sigma",
    "Υ"   => "\\Upsilon",
    "Φ"   => "\\Phi",
    "Ψ"   => "\\Psi",
    "Ω"   => "\\Omega",
    "∫∫∫" => "\\iiint",
    "(?<!∫)∫∫(?!∫)"  => "\\iint",
    "(?<!∫)∫(?!∫)"   => "\\int",
    "∂"   => "\\partial",
    "∇"   => "\\nabla",
    "×"   => "\\times",
    "÷"   => "\\div",
    "·"   => "\\cdot",
    "±"   => "\\pm",
    "→"   => "\\rightarrow",
    "←"   => "\\leftarrow",
    "⤳"   => "\\rightsquigarrow",
    "⇒"   => "\\Rightarrow",
    "⇐"   => "\\Leftarrow",
    "≠"   => "\\neq",
    "≤"   => "\\leq",
    "≥"   => "\\geq",
    "≈"   => "\\approx",
    "⊃"  => "\\supset",
    "⊂"  => "\\subset",
    "⊇"  => "\\supseteq",
    "⊆"  => "\\subseteq",
    "∖"  => "\\setminus",
    "⇔"  => "\\Leftrightarrow",
    "≡"  => "\\equiv",
    "ℵ"   => "\\aleph",
    "¬"   => "\\lnot",
    "∧"  => "\\land",
    "∨"  => "\\lor",
    "(?<! )∀"  => "\\forall",
    " ∀"  => "\\hspace{.2cm}\\forall",
    "∃"  => "\\exists",
    "∄"   => "\\nexists",
    "ℓ"   => "\\ell",
    "∞"   => "\\infty",
    "∈"  => "\\in",
    "∋"  => "\\ni",
    "~"  => "\\sim",
    "ℝ"  => "\\mathbb{R}",
    "ℕ"  => "\\mathbb{N}",
    "ℂ"  => "\\mathbb{C}",
    "ℤ"  => "\\mathbb{Z}",
    "ℚ"  => "\\mathbb{Q}",
    "𝔽"  => "\\mathbb{F}",
    "ø"  => "\\emptyset",
    "∅"  => "\\emptyset",
    "∘"  => "\\circ",
    "∑"  => "\\sum",
    "∏"  => "\\prod",
    "§"  => "\\S",
    "∪_" => "\\bigcup_",
    "∪(?!\_)"  => "\\cup",
    "∩_" => "\\bigcap_",
    "∩(?!\_)"  => "\\cap",
    "‹"  => "\\langle",
    "›"  => "\\rangle",
    "→"   => "\\rightarrow",
    "↑"  => "\\uparrow",
    "‣"   => "\\rightarrow", # TEMPORARY!
    "⇒"   => "\\Rightarrow",
    "≠"   => "\\neq",
    "≤"   => "\\leq",
    "≥"   => "\\geq",
    "≈"   => "\\approx",
    "♯"  => "{\\sharp}", 
    "♭"  => "{\\flat}",
    "♮"  => "{\\natural}" 
);
# I haven't done anything with this yet... future feature?
%replacements_math_only = (
    "(?<!\\\\)sin(?![A-Za-z0-9])" => "\\$1"
);

@rmfuns = ("sin", "cos", "tan", "arcsin", "arccos", "arctan", "asin",
           "acos", "atan", "sec", "csc", "cot", "ln", "log", "max",
           "min", "sup", "inf", "ker", "coker", "rng", "corng",
           "var", "rank", "rk", "exp", "erf");

%replacements = (
    "\\.\\.\\." => "\\ldots ",
    "…"         => "\\ldots ",
    "ä"         => '\"{a}',
    "ö"         => '\"{o}',
    "ü"         => '\"{u}',
    "LaTeX"     => "{\\LaTeX}",
    "St\\."       => "St.\\@",
    "Mr\\."       => "Mr.\\@",
    "Mrs\\."      => "Mrs.\\@",
    "Ms\\."       => "Ms.\\@",
    "Dr\\."       => "Dr.\\@",
    "Apt\\."      => "Apt.\\@",
    "Prof\\."     => "Prof.\\@",
    "(?<![𝆑𝆏𝆍𝆎𝆐])𝆑𝆑(?![𝆑𝆏𝆍𝆎𝆐])"  => "\\dynmark{ff}",
    "(?<![𝆑𝆏𝆍𝆎𝆐])𝆑(?![𝆑𝆏𝆍𝆎𝆐])"  => "\\dynmark{f}",
    "(?<![𝆑𝆏𝆍𝆎𝆐])𝆐𝆑(?![𝆑𝆏𝆍𝆎𝆐])"  => "\\dynmark{mf}",
    "(?<![𝆑𝆏𝆍𝆎𝆐])𝆐𝆏(?![𝆑𝆏𝆍𝆎𝆐])"  => "\\dynmark{mp}",
    "(?<![𝆑𝆏𝆍𝆎𝆐])𝆏(?![𝆑𝆏𝆍𝆎𝆐])"  => "\\dynmark{p}",
    "(?<![𝆑𝆏𝆍𝆎𝆐])𝆏𝆏(?![𝆑𝆏𝆍𝆎𝆐])"  => "\\dynmark{pp}",
    "(?<![𝆑𝆏𝆍𝆎𝆐])𝆑𝆏(?![𝆑𝆏𝆍𝆎𝆐])"  => "\\dynmark{fp}",
    "(?<![𝆑𝆏𝆍𝆎𝆐])𝆍𝆑𝆎(?![𝆑𝆏𝆍𝆎𝆐])"  => "\\dynmark{sfz}"
);

<===

** Grammar

This section defines the grammar/syntax used for the notes file
format.  It also outlines the format of the resulting parse tree,
which is used in the processing section.

===>
qr{
    <grammar: notes>
    <token: Paragraph>          <.bls> <lines> <.ble>
    <token: bls>               (?<=\n\n)
    <token: ble>               (?=\n\n)
    <token: partbls>           (?<=\n)
    <token: partble>           (?=\n)
    <token: line>              [^\n]+
    <token: lines>             (<[line]> \n)* <[line]>

    <token: BigMath>            <.bls> <optionallabel>? \$\$\$ \n <lines> \n \$\$\$ <.ble>
    <token: InlineMath>         <.math> <mathtext> <.math>
    <token: math>              (?<!\\)(?<!\$)\$
    <token: mathtext>          [^\$]+

    <token: Matrix2>            mat\[(<[matrixrow2]>,\s*\n*)* <[matrixrow2]>\]
    <token: matrixrow2>         \[(<[matrixcell2]>,\s*\n*)* <[matrixcell2]>\]
    <token: matrixcell2>        [^\s\]]+

    <token: Matrix>            mat\[(<[matrixrow]>;\s*\n*)* <[matrixrow]>\]
    <token: matrixrow>         (<[matrixcell]>[,\s]\s*)* <[matrixcell]>
    <token: matrixcell>        [^\s,;\]]+

    <token: Quote>             \" <quotetext> \"
    <token: quotetext>         [^"]+

    <token: RawLatex>          <.bls> \@\@\@ \n <lines> \n \@\@\@ <.ble>

    <token: Comment>           <.partbls> (<.begincomment> <[line]> \n)* <.begincomment> <[line]> <.partble> 
    <token: begincomment>      \#\s
    
    <token: Preformatted>      <.bls> (<.beginpreformatted> <[line]> \n)* <.beginpreformatted> <[line]> <.ble>
    <token: beginpreformatted> >\s

    <token: Important>         <.bls> <.beginimportant> <lines> <.ble>
    <token: beginimportant>    \->\s

    <token: Update>            <.bls> UPDATE \s \(<date=([0-9/]+)>\): <lines> <.ble>

    <token: Image>             <.bls> <optionallabel>? <.beginimage> <line> <.endimage> <optionaldesc>? <.ble>
    <token: beginimage>        (?<!\<) < (?!\<)
    <token: endimage>          (?<!\>) > (?!\>)

    <token: Section>           <.bls> <.beginsection> <line> <.partble>
    <token: beginsection>      \*\s

    <token: Subsection>        <.bls> <.beginsubsection> <line> <.partble>
    <token: beginsubsection>   \*\*\s

    <token: Subsubsection>     <.bls> <.beginsubsubsection> <line> <.partble>
    <token: beginsubsubsection> \*\*\*\s

    <token: Bullets>           <.bls> (<[bulletpoint]> <[bulletline]> \n)* <[bulletpoint]> <[bulletline]> <.ble>
    <token: bulletline>        <line> <[trailinglines]>*
    <token: trailinglines>     \n [ ]+ [^\- 0-9][^ ][^\n]+ | \n [ ]+ [a-z0-9]+[^\.][^\n]+
    <token: bulletpoint>       <bulletwhitespace> <bulletpointmark> \s
    <token: bulletpointmark>   [\-] | [0-9]{1,3}\.
    <token: bulletwhitespace>  [ \t]*

    <token: Table>             <.bls> <optionallabel>? (<[tableline]> \n)* <[tableline]> <optionaldesc>? <.ble>
    <token: tableline>         [+|] [^\n]+ [+|]

    <token: Property>          <.partbls> [ \t]* <name=([A-Za-z]+)> : [ \t]* <val=line> <.partble>

    <token: Link>              <.beginlink> <linkprefix>? <linktext> <linktitle>? <.endlink>
    <token: beginlink>         \[\[
    <token: endlink>           \]\]
    <token: linkprefix>        http:// | ftp:// | doi: | ref:
    <token: linktext>          [^\]]+
    <token: linktitle>         <.linktitledelim> <linktext>
    <token: linktitledelim>    \]\[

    <token: Code>              <.bls> <optionallabel>? <.delimitcode> \n <codelines> \n <.delimitcode> <.ble>
    <token: codeline>          [^\n]*
    <token: codelines>         (<[codeline]> \n)*? <[codeline]>
    <token: delimitcode>       [\<]?===[\>]?

    <token: Label>             \<\< <linklabeltext> \>\>
    <token: optionallabel>     <Label> \n
    <token: linklabeltext>     [^\>]+

    <token: Description>       \{ <descriptiontext>  \}
    <token: optionaldesc>      \n <Description>
    <token: descriptiontext>   [^\}]+


}xms;
<===

** Utility functions

These functions contain code that is reusable in multiple of the processing functions.

-> One thing to note about the design is that the "\ensuremath"
   function is necessary, even in math environments, and should not be
   removed.  It ensures that things will remain grouped together.  In
   theory, plain curly brackets could replace it in math environments.
   For instance, it makes sure "$2^\alpha\beta$" shows up properly.

===>
sub math_formatting {
    my $text = shift;
    do {
        no Regexp::Grammars;
        $text =~ s/(?<!_)_(-?[A-Za-z0-9]+)(?!\_)/\_{$1}/g; # Easy subscript/superscript words
        $text =~ s/\^(-?[A-Za-z0-9]+)(?!\^)/\^{$1}/g; 
        $text =~ s/√([A-Za-z0-9\\\{\}]+)/\\sqrt{$1}/g; # Square roots # FIXME the {} in there is a hack to get \sqrt{\pi} to work with \ensuremath
        $text =~ s/√\(([^\)]+)\)/\\sqrt{$1}/g; 
        $text =~ s/\<\</\\ll/g; # Much greater than/less than
        $text =~ s/\>\>/\\gg/g;
        $text =~ s/%/\\%/g; # Percent signs
        #$text =~ s/([0-9A-Za-z]+)\/([0-9A-Za-z]+)/\\frac{$1}{$2}/g; # Easy short fractions
        $text =~ s/\|\|(.*?)\|\|/\\norm\*{$1}/g; # Norm signs
        $text =~ s/\|(.*?)\|/\\abs\*{$1}/g; # Absolute value signs
        $text =~ s/\(/ \\left\( /g; #Math parentheses
        $text =~ s/\)/ \\right\) /g;
        $text =~ s/\*([^\* \n]+)\*/\\bm{$1}/gs; # Bold (matrix) text

        $funnames = join("|", @rmfuns); # Change sin -> \sin, ln -> \ln, etc.
        $text =~ s/(?<![\\A-Za-z0-9\"])($funnames)(?![A-Za-z0-9\"])/\\$1/g;

        # Moved this to the end to not interfere with putting "max" in quotes
        $text =~ s/\"([^\"]+)\"/\\text{$1}/gs; # Quotes make inline text automatically

    };
    return $text;
}

sub utf8_substitute {
    my $text = shift;
    my $inmathenv = shift; # Are we in the math environment already?
    do {
        no Regexp::Grammars;
        use Data::Dumper::Simple;
        @keys = keys %replacements_math;
        foreach (@keys) {
            $from = $_;
            $to = $replacements_math{$_};
            if ($inmathenv == 1) {
                $text =~ s/$from/\\ensuremath{$to}/sg;
            }
            else {
                $text =~ s/$from/\\ensuremath{$to}/sg;
            }
        }
        @keys = keys %replacements;
        foreach (@keys) {
            $from = $_;
            $to = $replacements{$_};
            $text =~ s/$from/$to/sg;
        }
        # Combining marks
        $text =~ s/(.)̃/\\ensuremath{\\tilde{$1}}/g;
        $text =~ s/(.)̂/\\ensuremath{\\hat{$1}}/g;
        $text =~ s/(.)⃗/\\ensuremath{\\vec{$1}}/g;
        $text =~ s/(.)̄/\\ensuremath{\\bar{$1}}/g; # Note, also called a macron
        $text =~ s/(.)̉/\\ensuremath{\\overset{?}{$1}}/g;
        $text =~ s/(.)̇/\\ensuremath{\\dot{$1}}/g;
    };

    return $text;
}

sub text_substitute {
    my $text = shift;
    do {
        no Regexp::Grammars;
        $text =~ s/\*\*([^ ].*?)\*\*/\*$1\*/gs; # Emphasized text - backward compatibility
        $text =~ s/(?<=\s)\*([^ \n][^\*]*[^ \n])\*/\\textbf{$1}/gs; # Bold text
        $text =~ s/(?<=\s)\/([^ \n][^\/]*[^ \n])\//\\textit{$1}/gs; # Italic text
        $text =~ s/(?<=\s)_([^ _\n][^_]*[^ _\n])_([ \n,!.:;)"'])/\\underline{$1}$2/gs; # Underline text
        $text =~ s/\%/\\\%/g; # Replace percent signs with real percent signs

    };
    return $text;
}

sub escape_latex {
    my $text = shift;
    do {
        no Regexp::Grammars;
        $text =~ s/\\/\\textbackslash /gs;
        $text =~ s/{/\\{/gs;
        $text =~ s/}/\\}/gs;
        $text =~ s/%/\\%/gs;
        $text =~ s/\_/\\\_/gs;
        $text =~ s/\#/\\\#/gs;
        $text =~ s/\^/\\\^/gs;
        $text =~ s/~/\\~/gs;
        $text =~ s/\$/\\\$/gs;
        $text =~ s/\&/\\\&/gs;
    };
    return $text;
}

sub create_absolute_link {
    my $text = shift;
    return "$filename.d/$text";
}

sub get_citation_from_doi {
    my $url = shift;
    my $output = "";
    # Resolve the DOI and write it to a text file
    $command = "curl -LH \"Accept: application/x-research-info-systems\" 'http://dx.doi.org/$url'";
    $bibcommand = "curl -LH \"Accept: text/bibliography; style=bibtex\" 'http://dx.doi.org/$url'";
    if ($properties{'Debug'} ne "True") {
        $output = `$command`;
        if ($properties{'Bibliography'} eq "auto") {
            $bibtex_entries .= `$bibcommand` . "\n";
        }
    }
    else {
        $output = `false`;
    }
    if ($? == 0) { # Fetch successful, exit status 0
        do {
            no Regexp::Grammars;
            my ($authorlast) = $output =~ /AU\s*-\s*([^,]+),/;
            my ($year) = $output =~ /PY\s*-\s*([0-9]{4})/;
            return "($authorlast, $year)";
        }
    }
    else {
        return $url;
    }
}            

<===

** Processing rule types

These functions are called whenever a match is found from the grammar.
The parse tree is their only argument.  They process the parse tree
and return valid LaTeX that represents the information from the parse
tree.

===>
sub proc_preformatted {
    my $tree = shift;
    my $line = join("\n", @{$tree->{Preformatted}->{line}});
    $verbtext = "verbatim";
    return "\\begin{$verbtext}\n$line\n\\end{$verbtext}";
}
sub proc_rawlatex {
    my $tree = shift;
    my $line = join("\n", @{$tree->{RawLatex}->{lines}->{line}});
    return $line;
}
sub proc_image {
    my $tree = shift;
    my $imgfile = create_absolute_link(${$tree}{Image}{line});
    my $text = "\\begin{center}\n";
    my $label = "";
    $text .= "\\includegraphicsscaled{$imgfile}\n";
    $text .= "\\end{center}\n";

    if (exists $tree->{Image}->{optionallabel}) {
        $label = $tree->{Image}->{optionallabel}->{Label}->{linklabeltext};
        $text = "\\hypertarget{" . $label . "}{}\n" . $text;
        $labeltexts{$label} = 'Figure';
        $labeltypes{$label} = 'fig';
    }

    if (exists $tree->{Image}->{optionaldesc}) {
        my $desc = $tree->{Image}->{optionaldesc}->{Description}->{descriptiontext};
        $desc = utf8_substitute($desc);
	    if ($label ne "") { $label = "\n\\label{fig:$label}"; }
        my $prefigure = "\\begin{figure}[h!]\n";
        my $postfigure = "\n\\caption{$desc}$label\n\\end{figure}";
	    $text = $prefigure . $text . $postfigure;
    }

    return $text;
}
sub proc_inlinemath {
    my $tree = shift;
    my $text = ${$tree}{InlineMath}{mathtext};
    $text = utf8_substitute($text);
    $text = math_formatting($text);
    return '$' . $text . '$';
}
sub proc_quotes {
    my $tree = shift;
    my $text = ${$tree}{Quote}{quotetext};
    $text = escape_latex($text);
    $text = utf8_substitute($text);
    return "\\texttt{\"\\textbf{" . $text . "}\"}";
}
sub proc_property {
    my $tree = shift;
    my $name = ${$tree}{Property}{name};
    my $val = ${$tree}{Property}{val};
    $properties{$name} = $val;
    return "\n";
}
sub proc_comment {
    my $tree = shift;
    my $line = join("\n", @{$tree->{Comment}->{line}});
    $line = utf8_substitute($line);
    if ($properties{'Type'} eq "Report") {
        return "";
    }
    return "\\sout{" . $line . "}";
}
sub proc_label {
    my $tree = shift;
    my $labelname = ${$tree}{Label}{linklabeltext};
    return "\\hypertarget{" . $labelname . "}{}";
}
sub proc_matrix {
    # use Data::Dumper::Simple;
    my $tree = shift;
    # print Dumper $tree;
    my @rows = @{$tree->{Matrix}->{matrixrow}};
    # Check for an augmented matrix
    my $endmatrix = "";
    my @firstrow = @{$tree->{Matrix}->{matrixrow}[0]->{matrixcell}};
    if ($firstrow[scalar(@firstrow)-2] eq "|") {
        # $latexmatrix = "\\ensuremath{\\left[\\begin{array}{" . "c"x(scalar(@firstrow)-2) . "|c} ";
        # $endmatrix = "\\end{array}\\right]}";
        $latexmatrix = "\\ensuremath{\\begin{bmatrix}[" . "c"x(scalar(@firstrow)-2) . "|c] ";
        $endmatrix = "\\end{bmatrix}}";
    }
    else {
        $latexmatrix = "\\ensuremath{\\begin{bmatrix} ";
        $endmatrix = "\\end{bmatrix}}";
    }
    for (my $i=0; $i<scalar(@rows); $i++) {
        my @cells = @{$tree->{Matrix}->{matrixrow}[$i]->{matrixcell}};
        for (my $j=0; $j<scalar(@cells)-1; $j++) {
            # Skip augmented matrix bars
            if ($j==scalar(@cells)-2 && $cells[$j] eq "|") {
                next;
            }
            my $cellval = $cells[$j];
            $cellval = utf8_substitute($cellval);
            $cellval = text_substitute($cellval);
            $cellval = math_formatting($cellval);
            $latexmatrix .= $cellval . " & ";
        }
        my $cellval = $cells[scalar(@cells)-1];
        $cellval = utf8_substitute($cellval);
        $cellval = text_substitute($cellval);
        $cellval = math_formatting($cellval);
        $latexmatrix .= $cellval . " \\\\";
    }
    $latexmatrix .= $endmatrix;
    return $latexmatrix;
}
sub proc_bigmath {
    my $tree = shift;
    my $line = join(" \\\\\n", @{$tree->{BigMath}->{lines}->{line}});
    my $firstline = ${$tree->{BigMath}->{lines}->{line}}[0];
    my $text = "";
    my $label = "";
    if (exists $tree->{BigMath}->{optionallabel}) {
        $label = $tree->{BigMath}->{optionallabel}->{Label}->{linklabeltext};
    }

    # Do some things to pretty it up
	$line =~ s/\\sum/\\displaystyle \\sum/g; # Pretty summations

    # Check to see if it has alignment informaiton or not

    # I don't know what I want to use the "\" at the end of the line syntax for
    # if($firstline =~ m/\\$/x) {
    #     do {no Regexp::Grammars; $line =~ s/\\ \\\\/\\\\/; };
    #     $text .= "\\begin{equation*}\\begin{split}\n";
    #     $text .= $line;
    #     $text .= "\n\\end{split}\\end{equation*}";
    # }
    if ($firstline =~ m/&/x) {
        do {no Regexp::Grammars; $line =~ s/\\ \\\\\n/ /g; }; # Allows \ syntax at end of line to continue
        $text .= "\\begin{align*}\n";
        $text .= $line;
        $text .= "\n\\end{align*}";
    }
    elsif ($label ne "") {
        $text .= "\\begin{equation}\n";
        $text .= "\\label{eq:" . $label . "}\n";
        $text .= $line;
        $text .= "\n\\end{equation}";
    }
    else {
        do {no Regexp::Grammars; $line =~ s/\\ \\\\\n/ /g; };
        $text .= "\\begin{gather*}\n";
        $text .= $line;
        $text .= "\n\\end{gather*}";
    }

    if ($label ne "") {
        $text = "\\hypertarget{" . $label . "}{}\n" . $text;
        $labeltexts{$label} = 'Equation';
        $labeltypes{$label} = 'eq';
    }
    $text = utf8_substitute($text);
    $text = math_formatting($text);
    return $text;
}
sub proc_important {
    my $tree = shift;
    my $start = "\\begin{center}\n\\fbox{\n\\begin{minipage}{.8\\textwidth}\n";
    my $end = "\n\\end{minipage}\n}\n\\end{center}";
    my $startr = "\\begin{center}\\begin{minipage}{.8\\textwidth} \\bf\n";
    my $endr = "\n\\end{minipage}\n\\end{center}\\vspace{.2cm}";
    my $line .= join("\n", @{$tree->{Important}->{lines}->{line}});

    $line = utf8_substitute($line);
    $line = text_substitute($line);
    if ($properties{'Type'} eq "Report") {
        return $startr . $line . $endr;
    }
    else {
        return $start . $line . $end;
    }
}


sub proc_update {
    my $tree = shift;
    my $text = "\\begin{description}\n";
    $text .= "\\item[Update ({$tree->{Update}->{date}})] \n";
    $text .= join("\n", @{$tree->{Update}->{lines}->{line}});
    $text .= "\n\\end{description}";
    $text = utf8_substitute($text);
    $text = text_substitute($text);
    return $text;
}
sub proc_section {
    my $tree = shift;
    my $text = ${$tree}{Section}{line};
    $text = utf8_substitute($text);
    $text = text_substitute($text);
    if ($properties{'Type'} eq "Report") {
        return "\\subsection{" . $text . "}";
    }
    return "\\section{" . $text . "}";
}
sub proc_subsection {
    my $tree = shift;
    my $text = ${$tree}{Subsection}{line};
    $text = utf8_substitute($text);
    $text = text_substitute($text);
    if ($properties{'Type'} eq "Report") {
        return "\\subsubsection{" . $text . "}";
    }
    return "\\subsection{" . $text . "}";
}
sub proc_subsubsection {
    my $tree = shift;
    my $text = ${$tree}{Subsubsection}{line};
    $text = utf8_substitute($text);
    $text = text_substitute($text);
    if ($properties{'Type'} eq "Report") {
        return "\\paragraph{" . $text . "}";
    }
    return "\\subsubsection{" . $text . "}";
}
sub proc_table {
    my $tree = shift;
    use Data::Dumper;
    my @lines = @{$tree->{Table}->{tableline}};
    my $tfmt = $lines[0];
    my $label = "";
    no Regexp::Grammars;
    $tfmt =~ s/\|$//;
    $tfmt =~ s/[^|!]//g;
    $tfmt =~ s/\|/l /g; # These will be left aligned
    $tfmt =~ s/\!/| /g; # Vertical bar

    my $text = "\\begin{tabular}{$tfmt}\n";
    for (my $i=0; $i<scalar(@lines); $i++) {
        my $curline = $lines[$i];
        if ($curline =~ /^\|[+-]+\|\s*$/) { # Detect an hline
            $text .= "\\hline\n";
        }
        else {
            if ($i==0) {
                $curline =~ s/![ ]*\|/\|/g;
            }
            $curline =~ s/^\|//;
            $curline =~ s/^(.+)\|\s*$/$1 \\\\ /;
            $curline =~ tr/\|/\&/;
            $text .= $curline . "\n";
        }
    }
    $text .= "\\end{tabular}";
    $text = utf8_substitute($text);
    $text = text_substitute($text);
    use Regexp::Grammars;

    if (exists $tree->{Table}->{optionallabel}) {
        $label = $tree->{Table}->{optionallabel}->{Label}->{linklabeltext};
        $text = $text . "\\hypertarget{" . $label . "}{}\n";
        $labeltexts{$label} = 'Table';
        $labeltypes{$label} = 'tab';
    }
    # Whether to use "h" or "h!" as position information depending on document type
    my $postype = "h!";
    if ($properties{'Type'} eq "Report") {
        $postype = "";
    }
    if (exists $tree->{Table}->{optionaldesc}) {
        my $desc = $tree->{Table}->{optionaldesc}->{Description}->{descriptiontext};
        $desc = utf8_substitute($desc);
	    if ($label ne "") { $label = "\\label{tab:$label}"; }
        my $prefigure = "\\begin{center}\n\\begin{table}[$postype]\n\\caption{$desc}$label\n";
        my $postfigure = "\n\\end{table}\n\\end{center}";
	    $text = $prefigure . $text . $postfigure;
    }
    return $text;
}
sub proc_bullets {
    my $tree = shift;
    my $currentlevel = 0;
    my $returnstring = "";
    # Each time we read a new item, if it has more whitespace than the
    # previous one, we'll push a new item onto the levelstack.  The
    # type will either be "enumerate" or "itemize".  Since Perl
    # doesn't have good tuple support, we'll store the spacing
    # associated with that level in spacingstack.
    my @levelstack = ();
    my @spacingstack = ();
    use Data::Dumper;
    my @lines = @{$tree->{Bullets}->{bulletline}};
    my @spaces = @{$tree->{Bullets}->{bulletpoint}};
    no Regexp::Grammars;
    for (my $i=0; $i<scalar(@lines); $i++) {
        my $wslen = length($spaces[$i]->{bulletwhitespace});
        if ($wslen > $currentlevel) {
            my $env = "";
            if ($spaces[$i]->{bulletpointmark} =~ /[0-9]/) {
                $env = "enumerate";
            } else {
                $env = "itemize"; 
            }
            push(@levelstack, $env);
            $returnstring .= "\\begin{$env}\n";
            push(@spacingstack, $currentlevel);
            $currentlevel = $wslen;
        }
        while ($wslen < $currentlevel) {
            my $env = pop(@levelstack);
            $returnstring .= "\\end{$env}\n";
            $currentlevel = pop(@spacingstack);
        }
        $line = $lines[$i]->{line};
        $line .= join(" ", @{$lines[$i]->{trailinglines}});
        $line = utf8_substitute($line);
        $line = text_substitute(" $line "); # Some things at the start of a line won't substitute properly without that space in there
        $returnstring .= " "x$wslen . "\\item " . $line . "\n";
    }
    my $env = "";
    while ($currentlevel > 0) {
        my $env = pop(@levelstack);
        $returnstring .= "\\end{$env}\n";
        $currentlevel = pop(@spacingstack);
    }
    if ($spaces[0]->{bulletpointmark} =~ /[0-9]/) {
        $env = "enumerate";
    } else {
        $env = "itemize"; 
    }
    use Regexp::Grammars;
    return "\\begin{$env}\n$returnstring\\end{$env}";
}
sub proc_link {
    my $tree = shift;
    $linkloc = $tree->{Link}->{linktext};
    if (exists $tree->{Link}->{linktitle}->{linktext}) {
        $linktitle = $tree->{Link}->{linktitle}->{linktext};
    }
    else {
        $linktitle = $linkloc;
    }
    if (exists $tree->{Link}->{linkprefix}) {
        if ($tree->{Link}->{linkprefix} eq "http://") {
            return "\\href{$linkloc}{$linktitle}";
        }
        elsif ($tree->{Link}->{linkprefix} eq "doi:") {
            if ($linktitle eq $linkloc) {
                $linktitle = get_citation_from_doi($linkloc);
            }
            return "\\href{http://dx.doi.org/$linkloc}{$linktitle}";
            # TODO make link text related to doi source with command: curl -D - -L -H "Accept: text/turtle" "http://dx.doi.org/10.1126/science.1157784"
        }
        elsif ($tree->{Link}->{linkprefix} eq "ref:") {
            return "\\hyperlink{$linkloc}{$linktitle}";
        }
    }
    use Data::Dumper::Simple;
    if (exists $labeltexts{$linkloc}) {
        $labeltype = $labeltypes{$linkloc};
        $labeltext = $labeltexts{$linkloc};
        if ($linktitle eq $linkloc) {
            $linktitle = "$labeltext \\ref{$labeltype:$linkloc}";
        }
        return "\\hyperlink{$linkloc}{$linktitle}";
    }
    if (do {no Regexp::Grammars; $linkloc =~ m/.*\.n/;}) {
        return "\\href{run:$linkloc.pdf}{$linktitle}";
    }
    return "\\href{run:$id.$suffix.d/$linkloc}{$linktitle}";
}
%linenos = ();
$labellast = $id;
sub proc_code {
    my $tree = shift;
    my $num_lines = @{$tree->{Code}->{codelines}->{codeline}};
    my $line = join("\n", @{$tree->{Code}->{codelines}->{codeline}});

    # Figure out the programming language by the extension
    %langs = (
        "c" => "C",
        "h" => "C",
        "cpp" => "C++",
        "hpp" => "C++",
        "php" => "PHP",
        "py" => "Python",
        "pl" => "Perl",
        "m" => "Octave",
        "gnuplot" => "Gnuplot",
        "sh" => "sh",
        "erl" => "erlang",
        "el" => "Lisp"
    );
    
    my $text = "";
    my $labelext = "";
    my $lstoptions = "";
    my $label = $labellast;
    # Find the label, if it exists, and use it to detect the language
    # this segment is in.  See extract-code.pl for logic.
    if (exists $tree->{Code}->{optionallabel}) {
        $label = $tree->{Code}->{optionallabel}->{Label}->{linklabeltext};
        $text = "\\hypertarget{" . $label . "}{}\n" . $text;
        $labellast = $label;
    }
    do {
        no Regexp::Grammars;
        if ($label =~ m/\.([A-Za-z0-9]+)$/) {
            $labelext = $1;
        }
    };

    # Determine listings options
    if (exists $langs{$labelext}) {
        $lstoptions .= "language=$langs{$labelext},";
    }
    elsif (exists $langs{$ext}) {
        $lstoptions .= "language=$langs{$ext},";
    }
    # Numbering
    if (exists $linenos{$label}) {
        $lstoptions .= "firstnumber=$linenos{$label},";
        $linenos{$label} += $num_lines;
    }
    else {
        $lstoptions .= "firstnumber=1,";
        $linenos{$label} = $num_lines+1;
    }
    $from = $linenos{$label} - $num_lines;
    $to = $linenos{$label}-1;
    $lstoptions .= "title={\\tiny \\hfill $label, $from--$to},backgroundcolor=\\color\{lightgray\},frame=single,framerule=0pt,";


    if ($labelext eq "") {
        $verbtext = "verbatim";
        $text .= "\\begin{$verbtext}";
        $text .= $line;
        $text .= "\n\\end{$verbtext}";
    }
    else {
        $lstlistingtext = "lstlisting";
        $text .= "\\begin{$lstlistingtext}[$lstoptions]\n";
        $text .= $line;
        $text .= "\n\\end{$lstlistingtext}";
    }

    return $text;
}

<===

The "@rules" variable holds all of the information necessary to detect
and parse each token and aspect of the notes file syntax.  "regex"
contains a regular expression that will detect a particular feature
and generate its parse tree, and "proc" contains the name of the
function that can process the parse tree (from the section above).

-> The order of the elements in "@rules" matters!  Those that are
first in the list are processed first.  Hence, it is important to put
the rules that need verbatim text first, since no other tokens should
be detected within these.

===>
$inlinemath   = qr{ <extends: notes> <InlineMath> }xms;
$quotes       = qr{ <extends: notes> <Quote> }xms;
$matrix       = qr{ <extends: notes> <Matrix> }xms;
$bigmath      = qr{ <extends: notes> <BigMath> }xms;
$rawlatex     = qr{ <extends: notes> <RawLatex> }xms;
$comment      = qr{ <extends: notes> <Comment> }xms;
$preformatted = qr{ <extends: notes> <Preformatted> }xms;
$important    = qr{ <extends: notes> <Important> }xms;
$update       = qr{ <extends: notes> <Update> }xms;
$image        = qr{ <extends: notes> <Image> }xms;
$section      = qr{ <extends: notes> <Section> }xms;
$subsection   = qr{ <extends: notes> <Subsection> }xms;
$subsubsection= qr{ <extends: notes> <Subsubsection> }xms;
$bullets      = qr{ <extends: notes> <Bullets> }xms;
$table        = qr{ <extends: notes> <Table> }xms;
$property     = qr{ <extends: notes> <Property> }xms;
$label        = qr{ <extends: notes> <Label> }xms;
$link         = qr{ <extends: notes> <Link> }xms;
$code         = qr{ <extends: notes> <Code> }xms;

@rules = (
    {regex => $code, proc => 'proc_code'}, # Must go first to avoid quotation mark conflicts
    {regex => $preformatted, proc => 'proc_preformatted'}, # Go second to make sure all symbols are captured
    {regex => $rawlatex, proc => 'proc_rawlatex'}, # Do this next to make sure latex is preserved
    {regex => $image, proc => 'proc_image'},
    {regex => $property, proc => 'proc_property'},
    {regex => $matrix, proc => 'proc_matrix'},
    {regex => $bigmath, proc => 'proc_bigmath'},
    {regex => $inlinemath, proc => 'proc_inlinemath'},
    {regex => $quotes, proc => 'proc_quotes'},
    {regex => $comment, proc => 'proc_comment'},
    {regex => $important, proc => 'proc_important'},
    {regex => $update, proc => 'proc_update'},
    {regex => $section, proc => 'proc_section'},
    {regex => $subsection, proc => 'proc_subsection'},
    {regex => $subsubsection, proc => 'proc_subsubsection'},
    {regex => $table, proc => 'proc_table'},
    {regex => $link, proc => 'proc_link'}, # Must go after table, figure, and bigmath to make sure references work
    {regex => $bullets, proc => 'proc_bullets'}, #Must go after link to make sure links in bullets are handled
    {regex => $label, proc => 'proc_label'});
<===

** Prepare the content

Here, we process each type of token/section, listed above.

Iterate through each of the rules defined in @rules.  Look for the
regex, and for each instance it is found, pass a reference of the
Regexp::Grammars parse tree to the appropriate proc function.

===>
$tokenid = 0;
for(my $i=0; $i<scalar(@rules); $i++) {
    while (1) {
        if ($content =~ s/$rules[$i]{regex}/___TOKEN${tokenid}___/) {
            # We need to pass the result list by reference to the
            # function specified in @rules
            $text = &{$rules[$i]{proc}}(\%/);
            # Add the processed text to the @tokens array
            push(@tokens, $text);
            $tokenid++;
        }
        else {last;}
    }
}
<===

Replace all of the tokens in the string with the appropriate token
from @tokens.  Note that there is a bug in Regexp::Grammars, so we have
to disable it while doing substitutions.

===>
no Regexp::Grammars; 

# Replace the special characters
$content = utf8_substitute($content);
$content = text_substitute($content);

# Replace all the ___TOKENxxx___ symbols with the actual tokens
while ($content =~ s/___TOKEN([0-9]+)___/$tokens[$1]/g) {}
<===

** Header and Footer

"header" and "footer" define the top and bottom of the output LaTeX
document.  This is for initialization such as the begindocument line.

===>
if ($properties{'Type'} eq "Report") {
    $fontsize = "12pt";
}
else {
    $fontsize = "10pt";
}

$header = "\\documentclass[" . $fontsize . "]{article}
\\usepackage[pdftex]{graphicx}
\\usepackage{epstopdf}
\\epstopdfsetup{outdir=/tmp/}
\\usepackage[colorlinks=true,filecolor=blue]{hyperref}\n";

if (exists $properties{'Project'}) { $subject = "ID: $id, Project: $properties{'Project'}"; }
else { $subject = "ID: $id"; }
$header .= "\\hypersetup
{
    pdftitle={$title},
    pdfauthor={Max Shinn},
    pdfcreator={TeX},
    pdfsubject={$subject},
}\n";

$header .= "\\usepackage{underscore}
\\usepackage{grffile}
\\usepackage{ulem}
\\usepackage{gensymb}
\\usepackage{amsmath}
\\usepackage{amssymb}
\\renewcommand*\\rmdefault{ppl}\\normalfont\\upshape

\\usepackage{mathtools}
\\usepackage{bm} % For typesetting matrices
\\setcounter{MaxMatrixCols}{30} % Longer matrices

% Dynamic markings, from http://tex.stackexchange.com/questions/111788/how-to-typeset-music-dynamics-within-normal-text
\\newcommand{\\muskern}{\\kern-.15ex } % what MusiXTeX does
\\makeatletter
\\newcommand\\dynmark[1]{{\\normalfont\\bfseries\\itshape
  \\\@tfor\\next:=#1\\do{\\put\@muskern\\next}\\/}}
\\newcommand{\\put\@muskern}{\\let\\put\@muskern\\muskern}
\\makeatother

% Don't let images get too big
\\usepackage{calc}
\\newlength{\\imgwidth}
\\newcommand\\includegraphicsscaled[1]{%
    \\settowidth{\\imgwidth}{\\includegraphics{#1}}%
    \\setlength{\\imgwidth}{\\minof{\\imgwidth}{\\textwidth}}%
    \\includegraphics[width=\\imgwidth,keepaspectratio]{#1}%
}


% \\usepackage{enumitem} % Higher depth for bullet points
% \\setlistdepth{9}
% \\setlist[itemize,1]{label=\$\\bullet\$}
% \\setlist[itemize,2]{label=\$\\bullet\$}
% \\setlist[itemize,3]{label=\$\\bullet\$}
% \\setlist[itemize,4]{label=\$\\bullet\$}
% \\setlist[itemize,5]{label=\$\\bullet\$}
% \\setlist[itemize,6]{label=\$\\bullet\$}
% \\setlist[itemize,7]{label=\$\\bullet\$}
% \\setlist[itemize,8]{label=\$\\bullet\$}
% \\setlist[itemize,9]{label=\$\\bullet\$}
% \\renewlist{itemize}{itemize}{9}

\\DeclarePairedDelimiter\\abs{\\lvert}{\\rvert}
\\DeclarePairedDelimiter\\norm{\\lVert}{\\rVert}
";

if ($properties{'Type'} eq "Report") {
    $header .= "\n\\usepackage{fullpage}\n";
}

if ($properties{'Spacing'} eq "Double") {
    $header .= "\n\\usepackage{setspace}\n\n";
}

if (exists $properties{'Bibliography'}) {
    $header .= "\n\\usepackage{natbib}\n\n";
}

$header .= "
\\usepackage{listings}
\\usepackage{color}
\\usepackage[font=small,labelfont=bf]{caption}
%\\captionsetup{belowskip=12pt,aboveskip=4pt} %table spacing for captions
 
\\definecolor{dkgreen}{rgb}{0,0.6,0}
\\definecolor{gray}{rgb}{0.5,0.5,0.5}
\\definecolor{mauve}{rgb}{0.58,0,0.82}
\\definecolor{lightgray}{gray}{0.97}

\\DeclareMathOperator{\\rank}{rank}
\\DeclareMathOperator{\\rk}{rk}
\\DeclareMathOperator{\\var}{var}
\\DeclareMathOperator{\\cov}{cov}
\\DeclareMathOperator{\\coker}{coker}
\\DeclareMathOperator{\\rng}{rng}
\\DeclareMathOperator{\\corng}{corng}
\\DeclareMathOperator{\\erf}{erf}

% Augmented matrices
\\makeatletter
\\renewcommand*\\env\@matrix[1][*\\c\@MaxMatrixCols c]{%
  \\hskip -\\arraycolsep
  \\let\\\@ifnextchar\\new\@ifnextchar
  \\array{#1}}
\\makeatother

\\lstset{
  basicstyle=\\footnotesize,
  firstnumber=last,
  numbers=left,                   % where to put the line-numbers
  numberstyle=\\tiny\\color{gray},% the style that is used for the line-numbers
  stepnumber=1,                   % the step between two line-numbers. If it's 1, each line 
                                  % will be numbered
  numbersep=5pt,                  % how far the line-numbers are from the code
  backgroundcolor=\\color{white}, % choose the background color. You must add \usepackage{color}
  showspaces=false,               % show spaces adding particular underscores
  showstringspaces=false,         % underline spaces within strings
  showtabs=false,                 % show tabs within strings adding particular underscores
  tabsize=2,                      % sets default tabsize to 2 spaces
  captionpos=b,                   % sets the caption-position to bottom
  breaklines=true,                % sets automatic line breaking
  breakatwhitespace=false,        % sets if automatic breaks should only happen at whitespace
  keywordstyle=\\color{blue},     % keyword style
  commentstyle=\\color{dkgreen},  % comment style
  stringstyle=\\color{mauve},     % string literal style
  morekeywords={*,...}            % if you want to add more keywords to the set
}
\\hypersetup{
%    colorlinks,%
    citecolor=black,%
    filecolor=black,%
    linkcolor=black,%
    urlcolor=black
}

\\setcounter{secnumdepth}{0} % No numbering of sections

\\begin{document}
";

if ($properties{'Spacing'} eq "Double") {
    $header .= "\n\\doublespacing\n";
}
if (exists $properties{'Bibliography'}) {
    $header .= "\n\\bibliographystyle{cbe}\n\n";
}

$header .= "
\\begin{center}
\\section*{$title}
\\end{center}
";

if ($properties{'Type'} eq "Note") {
    $header .= "\n\\hfill {\\large \\textit{ID: $id}}\n";
    if (exists $properties{'Project'}) {
        $header .= "\\hfill {\\large \\textit{Project: $properties{'Project'}}} \\\\ \n";
    }
}
elsif ($properties{'Type'} eq "Report") {
    $header .= "\n \\begin{flushright} \n{\\large $properties{'By'}} \\\\ \n";
    if (exists $properties{'Course'}) {
        $header .= "{\\large $properties{'Course'}} \\\\ \n";
    }
    if (exists $properties{'Instructor'}) {
        $header .= "{\\large $properties{'Instructor'}} \\\\ \n";
    }
    if (exists $properties{'Opened'}) {
        $header .= "{\\large $properties{'Opened'}} \\\\ \n";
    }
    else {
        $header .= "\\hfill {\\large $properties{'Closed'}} \\\\ \n";
    }
    $header .= "\\end{flushright}\n";
}

$footer = "";
<===

Include a signature.  "Maxwell Shinn" is the default name, but this
can be overridden by setting the "By" property.

===>
die "Note never opened.  Add 'Opened: [date]' to the document header." unless exists $properties{'Opened'};

if ($properties{'Type'} eq "Note") {
    $footer .= "\n\n\\hspace{1in}\n\n";
    if (exists $properties{'Closed'}) {
        $footer .= "\\hfill ---$properties{'By'},~~\\textit{$properties{'Opened'}--$properties{'Closed'}}\n\n";
    }
    else {
        $footer .= "\\hfill ---$properties{'By'},~~\\textit{$properties{'Opened'}}\n\n";
    }
}
<===

Optionally include a bibliography.

===>
if (exists $properties{'Bibliography'} && $properties{'Bibliography'} ne "auto") {
    $bib = create_absolute_link($properties{'Bibliography'});
    $footer .= "\n\\bibliography{$bib}\n\n";
}
elsif (exists $properties{'Bibliography'} && $properties{'Bibliography'} eq "auto") {
    # First clean the entries to remove underscores, and then write to a file.  We're still
    # using "no Regexp::Grammars" and thus don't have to specify it again.
    $bibtex_entries =~ tr/_/-/;
    $bibtex_entries =~ s/\},/\},\n/g;
    $bib = "$filename.bib";
    open HANDLE, ">",  $bib;
    print HANDLE $bibtex_entries;
    close HANDLE;
    $footer .= "\n\\nocite{*}\\bibliography{$bib}\n\n"
}
<===

** Closing

Close the document and output the results

===>
$footer .= "\\end{document}\n";


print $header, "\n", $content, "\n", $footer
<===

* Extracting source code

The following will extract source code from any notes file and export
it to several individual files.

** Initialization

Basic header and some modules.

<<extract-code.pl>>
===>
#!/usr/bin/perl -w

use Regexp::Grammars;
use File::Basename;
<===

Load the file contents into the "$contents" variable.

===>
$filename = $ARGV[0];
local $/ = undef;
open(FILE, $filename);
$content = <FILE>;
close(FILE);
<===

Now find the filename, path, etc.

===>
$id = "";
do { 
    no Regexp::Grammars; 
    if ($filename =~ m/^.*?([^\/\\]+)\.([nr])$/) {
        $id = $1;
    }
    else {
        die "Invalid file name.";
    }
};
$path = (fileparse($filename))[1];
<===

The second optional command line parameter allows you to specify one
of the source files to be displayed as output.  Even if it is
displayed as output, it will still be written to the correct file.

===>
if (exists $ARGV[1]) {
    $onlyshow = $ARGV[1];
}
else {
    $onlyshow = "";
}
<===

** Grammar

I need to duplicate everything below because Perl is dumb and I can't
seem to get the scoping to work correctly within "Regexp::Grammars"
when I include it as an external file.

===>
qr{
    <grammar: notes>
    <token: Paragraph>          <.bls> <lines> <.ble>
    <token: bls>               (?<=\n\n)
    <token: ble>               (?=\n\n)
    <token: partbls>           (?<=\n)
    <token: partble>           (?=\n)
    <token: line>              [^\n]+
    <token: lines>             (<[line]> \n)* <[line]>
    <token: Code>              <.bls> <optionallabel>? <.delimitcode> \n <codelines> \n <.delimitcode> <.ble>
    <token: codeline>          [^\n]*
    <token: codelines>         (<[codeline]> \n)*? <[codeline]>
    <token: delimitcode>       [\<]?===[\>]?

    <token: Label>             \<\< <linklabeltext> \>\>
    <token: optionallabel>     <Label> \n
    <token: linklabeltext>     [^\>]+

}xms;

$code = qr{ <extends: notes> <Code> }xms;
<===

** Extracting the source code

Now we actually extract the source code.  Each label is used as the
name of the source code file.  The file is read sequentially.  If
there is a label, this label is the name of the file.  If there is not
a label, the most recent label is used (i.e. the label that was used
on the previous code block).  If the first one does not have a label,
the label is taken from the file name of the notes file.

===>
%sourcetexts = ();
$last = "";
while (1) {
    if ($content =~ s/$code//) {
        my $line = join("\n", @{$/{Code}->{codelines}->{codeline}}) . "\n";
        if (exists $/{Code}->{optionallabel}) {
            my $label = $/{Code}->{optionallabel}->{Label}->{linklabeltext};
            if (exists $sourcetexts{$label}) {
                $sourcetexts{$label} .= $line;
                $last = $label;
            }
            else {
                $sourcetexts{$label} = $line;
                $last = $label;
            }
        }
        elsif ($last ne "") {
            $sourcetexts{$last} .= $line;
        }
        else {
            $sourcetexts{$id} = $line;
            $last = $id;
        }
    }
    else {last;}
}
<===

Now actually print them all out to files.

===>
while(($k,$v)=each(%sourcetexts)) {
    open HANDLE, ">",  $path . "/" . $k;
    print HANDLE $v;
    close HANDLE;
    if ($onlyshow eq $k) {
        print $v;
    }
}
<===



